@@ -43,11 +43,11 @@ typedef struct
 } tree;
 
 extern CardProperty 
-empty_CardProperty(void)
+card_prop_empty(void)
 {
 	CardProperty prop;
 	
-	prop.used = 0;
+	prop.used = FALSE;
 	
 	prop.type = PROP_NONE;
 	prop.encod = ENC_7BIT;
@@ -66,12 +66,24 @@ empty_CardStrProperty(void)
 {
 	CardStrProperty strprop;
 	
-	strprop.prop = empty_CardProperty();
+	strprop.prop = card_prop_empty();
 	strprop.str = NULL;
 	
 	return strprop;
 }
 
+/* Intended to check asserts. */
+extern int card_check_prop (CardProperty prop)
+{
+	if (((prop.used == FALSE) || (prop.used == TRUE)) &&
+	    ((prop.type >= PROP_NONE) && (prop.type <= PROP_LAST)) &&
+	    ((prop.encod >= ENC_NONE) && (prop.encod <= ENC_LAST)) &&
+	    ((prop.value >= VAL_NONE) && (prop.value <= VAL_LAST)))
+	  return TRUE;
+	    
+	return FALSE;
+}
+
 extern void
 card_prop_free(CardProperty prop)
 {
@@ -101,14 +113,10 @@ card_new(void)
 	c->photo.size = 0;
 	c->photo.data = NULL;
 	c->bday.year = c->bday.month = c->bday.day = 0;
-/* NOT USED
 	c->deladdr.l  = NULL;
 	c->dellabel.l = NULL;
-*/
-	c->postal.l   = NULL;
 	c->phone.l    = NULL;
-	c->email.type  = EMAIL_INET;
-	c->email.address  = NULL;
+	c->email.l    = NULL;
 	c->timezn.sign  = 1;
 	c->timezn.hours = c->timezn.mins = 0;
 	c->geopos.lon   = c->geopos.lat  = 0;
@@ -133,20 +141,17 @@ card_new(void)
 	
 	c->name.prop   = c->photo.prop = c->bday.prop    = c->timezn.prop   = 
 	c->geopos.prop = c->logo.prop  = c->org.prop     = c->rev.prop      =
-/*	c->sound.prop  = c->key.prop   = c->deladdr.prop = c->dellabel.prop =*/
-	c->sound.prop  = c->key.prop   = c->postal.prop =
-	c->phone.prop  = c->email.prop = c->prop = empty_CardProperty();
+	c->sound.prop  = c->key.prop   = c->deladdr.prop = c->dellabel.prop =
+	c->phone.prop  = c->email.prop = c->prop = card_prop_empty();
 	
 	c->prop.type = PROP_CARD;
 	c->fname.prop.type = PROP_FNAME;
 	c->name.prop.type = PROP_NAME;
 	c->photo.prop.type = PROP_PHOTO;
 	c->bday.prop.type = PROP_BDAY;
-/* NOT USED
+	
 	c->deladdr.prop.type = PROP_DELADDR_LIST;
 	c->dellabel.prop.type = PROP_DELLABEL_LIST;
-*/
-	c->postal.prop.type = PROP_POSTADDR_LIST;
 	c->phone.prop.type = PROP_PHONE_LIST;
 	c->email.prop.type = PROP_EMAIL_LIST;
 	c->mailer.prop.type = PROP_MAILER;
@@ -200,21 +205,21 @@ card_free(Card *crd)
 	MY_FREE(crd->uid.str); card_prop_free(crd->uid.prop);
 
 	/* address is a little more complicated */
-	card_prop_free(crd->postal.prop);
-	while ((l = crd->postal.l)) {
-	    CardPostAddr *p;
+	card_prop_free(crd->deladdr.prop);
+	while ((l = crd->deladdr.l)) {
+		int i;
+		CardDelAddr *p;
 
-	    p = (CardPostAddr *)(l->data);
-	    card_prop_free(p->prop);
-	    MY_FREE(p->street1);
-	    MY_FREE(p->street2);
-	    MY_FREE(p->city);
-	    MY_FREE(p->state);
-	    MY_FREE(p->zip);
-	    MY_FREE(p->country);
-	    crd->postal.l = g_list_remove_link(crd->postal.l, l);
-	    g_list_free(l);
+		p = (CardDelAddr *)(l->data);
+		card_prop_free(p->prop);
+		
+		for (i = 0; i < DELADDR_MAX; i++)
+		  MY_FREE (p->data[i]);
+		
+		crd->deladdr.l = g_list_remove_link(crd->deladdr.l, l);
+		g_list_free(l);
 	}
+	
 	free(crd);
 }
 
@@ -386,8 +391,8 @@ get_CardProperty(VObject *o)
 	char *the_str;
 	CardProperty prop;
 
-	prop = empty_CardProperty();
-	prop.used = 1;
+	prop = card_prop_empty();
+	prop.used = TRUE;
 	
 	if (has (o, VCValueProp)) {
 		if (has (vo, VCContentIDProp))
@@ -504,55 +509,54 @@ strtoCardBDay(char *str)
 	return bday;
 }
 
-static CardPostAddr *
-get_CardPostAddr(VObject *o)
+static CardDelAddr *
+get_CardDelAddr(VObject *o)
 {
 	VObject *vo;
 	char *the_str;
-	CardPostAddr *addr;
+	CardDelAddr *addr;
+	int i;
 	
-	addr = malloc(sizeof(CardPostAddr));
+	addr = malloc(sizeof(CardDelAddr));
 
-	addr->street1 = addr->street2 = addr->city = addr->state = NULL;
-	addr->country = addr->zip = NULL;
+	for (i = 0; i < DELADDR_MAX; i++)
+	  addr->data[i] = NULL;
+	
 	addr->type    = get_addr_type(o);
 	addr->prop    = get_CardProperty(o);
 
-/* NOT IMPLEMENTED CURRENTLY
 	if (has (o, VCPostalBoxProp)) {
 		addr->data[PO] = g_strdup(str_val(vo));
 		free(the_str);
 	}
-*/
 	if (has (o, VCExtAddressProp)) {
-		addr->street2 = g_strdup(str_val(vo));
+		addr->data[EXT] = g_strdup(str_val(vo));
 		free(the_str);
 	}
 	if (has (o, VCStreetAddressProp)) {
-		addr->street1 = g_strdup(str_val(vo));
+		addr->data[STREET] = g_strdup(str_val(vo));
 		free(the_str);
 	}
 	if (has (o, VCCityProp)) {
-		addr->city = g_strdup(str_val(vo));
+		addr->data[CITY] = g_strdup(str_val(vo));
 		free(the_str);
 	}
 	if (has (o, VCRegionProp)) {
-		addr->state = g_strdup(str_val(vo));
+		addr->data[REGION] = g_strdup(str_val(vo));
 		free(the_str);
 	}
 	if (has (o, VCPostalCodeProp)) {
-		addr->zip = g_strdup(str_val(vo));
+		addr->data[CODE] = g_strdup(str_val(vo));
 		free(the_str);
 	}
 	if (has (o, VCCountryNameProp)) {
-		addr->country = g_strdup(str_val(vo));
+		addr->data[COUNTRY] = g_strdup(str_val(vo));
 		free(the_str);
 	}
 	
 	return addr;
 }
 
-/* NOT USED	
 static CardDelLabel *
 get_CardDelLabel(VObject *o)
 {
@@ -568,7 +572,6 @@ get_CardDelLabel(VObject *o)
 	free(the_str);
 	return dellabel;
 }
-*/
 
 static CardPhone *
 get_CardPhone(VObject *o)
@@ -586,17 +589,16 @@ get_CardPhone(VObject *o)
 	return ret;
 }
 
-static CardEMail
+static CardEMail *
 get_CardEMail(VObject *o)
 {
-	CardEMail ret;
+	CardEMail *ret;
 	char *the_str; 
 	
-/*	ret = malloc(sizeof(CardEMail)); */
-	ret.address = NULL;
-	ret.type = get_email_type(o);
-	ret.address = g_strdup(str_val(o));
-	ret.prop = get_CardProperty(o);
+	ret = malloc(sizeof(CardEMail)); 
+	ret->type = get_email_type(o);
+	ret->data = g_strdup(str_val(o));
+	ret->prop = get_CardProperty(o);
 	
 	free(the_str);
 	
@@ -618,7 +620,7 @@ strtoCardTimeZone(char *str)
 	tz.hours = 0;
 	tz.mins = 0;
 	
-	s[3] = 0;
+	s[2] = 0;
 	if (strlen(str) > 2) {
 		s[0] = str[0];
 		s[1] = str[1];
@@ -724,7 +726,7 @@ strtoCardRev(char *str)
 		str[len] = 0;
 	}
 	  
-	s[3] = 0;
+	s[2] = 0;
 	t = strchr(str, 'T');
 	if (t) {                            /* Take the Time */
 		*t = 0;
@@ -915,15 +917,15 @@ card_create_from_vobject (VObject *vcrd)
 			crd->bday = strtoCardBDay(str_val(o));
 			free(the_str);
 			break;
-		 case PROP_POSTADDR:
+		 case PROP_DELADDR:
 			{
-				CardPostAddr *c;
-				c = get_CardPostAddr(o);
+				CardDelAddr *c;
+				c = get_CardDelAddr(o);
 				prop = &c->prop;
-				crd->postal.l = g_list_append(crd->postal.l, c);
+				crd->deladdr.l = g_list_append(crd->deladdr.l, c);
 			}
 			break;
-/* NOT IMPLEMENTED YET
+			
 		 case PROP_DELLABEL:
 			{
 				CardDelLabel *c;
@@ -932,7 +934,7 @@ card_create_from_vobject (VObject *vcrd)
 				crd->dellabel.l = g_list_append(crd->dellabel.l, c);
 			}
 			break;
-*/
+			
 		 case PROP_PHONE:
 			{
 				CardPhone *c;
@@ -943,11 +945,10 @@ card_create_from_vobject (VObject *vcrd)
 			break;
 		 case PROP_EMAIL:
 			{
-				/*CardEMail *c;*/
-				/*c = get_CardEMail(o);*/
-				/*prop = &c->prop;*/
-			        prop = &crd->email.prop;
-				crd->email = get_CardEMail(o);
+				CardEMail *c;
+				c = get_CardEMail(o);
+				prop = &c->prop;
+				crd->email.l = g_list_append(crd->email.l, c);
 			}
 			break;
 		 case PROP_MAILER:
@@ -1273,12 +1274,14 @@ card_convert_to_vobject(Card *crd)
 		add_strProp(vprop, VCNameSuffixesProp, crd->name.suffix);
 		add_CardProperty(vprop, &crd->name.prop);
 	}
+	
 	if (crd->photo.prop.used) {
 		vprop = addPropSizedValue(vobj, VCPhotoProp, 
 					  crd->photo.data, crd->photo.size);
 		add_PhotoType(vprop, crd->photo.type);
 		add_CardProperty(vprop, &crd->photo.prop);
 	}
+	
 	if (crd->bday.prop.used) {
 		char *date_str;
 		
@@ -1287,29 +1290,28 @@ card_convert_to_vobject(Card *crd)
 		free(date_str);
 		add_CardProperty(vprop, &crd->bday.prop);
 	}
-	if (crd->postal.l) {
+	
+	if (crd->deladdr.l) {
 		GList *node;
 		
-		for (node = crd->postal.l; node; node = node->next) {
-			CardPostAddr *postaddr = (CardPostAddr *) node->data;
+		for (node = crd->deladdr.l; node; node = node->next) {
+			CardDelAddr *deladdr = (CardDelAddr *) node->data;
 			
-			if (postaddr->prop.used) {
+			if (deladdr->prop.used) {
 				vprop = addProp(vobj, VCAdrProp);
-				add_AddrType(vprop, postaddr->type);
-/* NOT IMPLEMENTED
-			        add_strProp(vprop, VCPostalBoxProp, postaddr->data[PO]);
-*/
-				add_strProp(vprop, VCExtAddressProp,postaddr->street2);
-				add_strProp(vprop, VCStreetAddressProp,postaddr->street1);
-				add_strProp(vprop, VCCityProp, postaddr->city);
-				add_strProp(vprop, VCRegionProp, postaddr->state);
-				add_strProp(vprop, VCPostalCodeProp, postaddr->zip);
-				add_strProp(vprop, VCCountryNameProp, postaddr->country);
-				add_CardProperty(vprop, &postaddr->prop);
+				add_AddrType(vprop, deladdr->type);
+			        add_strProp(vprop, VCPostalBoxProp, deladdr->data[PO]);
+				add_strProp(vprop, VCExtAddressProp,deladdr->data[EXT]);
+				add_strProp(vprop, VCStreetAddressProp,deladdr->data[STREET]);
+				add_strProp(vprop, VCCityProp, deladdr->data[CITY]);
+				add_strProp(vprop, VCRegionProp, deladdr->data[REGION]);
+				add_strProp(vprop, VCPostalCodeProp, deladdr->data[CODE]);
+				add_strProp(vprop, VCCountryNameProp, deladdr->data[COUNTRY]);
+				add_CardProperty(vprop, &deladdr->prop);
 			}
 		}
 	}
-/* NOT IMPLEMENTED
+	
 	if (crd->dellabel.l) {
 		GList *node;
 		
@@ -1322,7 +1324,7 @@ card_convert_to_vobject(Card *crd)
 			add_CardProperty(vprop, &dellabel->prop);
 		}
 	}
-*/
+	
 	if (crd->phone.l) {
 		GList *node;
 		
@@ -1339,30 +1341,23 @@ card_convert_to_vobject(Card *crd)
 		}
 	}
 
-/* NO LONGER USE A LIST FOR EMAIL
 	if (crd->email.l) {
 		GList *node;
 		
 		for (node = crd->email.l; node; node = node->next) {
 			CardEMail *email = (CardEMail *) node->data;
 			
-			vprop = add_strProp(vobj, VCEmailAddressProp, 
-					    email->data);
-			add_EMailType(vprop, email->type);
-			add_CardProperty(vprop, &email->prop);
+			if (email->prop.used) {
+				vprop = add_strProp(vobj, VCEmailAddressProp, 
+						    email->data);
+				add_EMailType(vprop, email->type);
+				add_CardProperty(vprop, &email->prop);
+			}
 		}
 	}
-*/
-/* Instead we have a single email field */
-	if (crd->email.prop.used && crd->email.address) {
-	    vprop = add_strProp(vobj, VCEmailAddressProp, crd->email.address);
-			add_EMailType(vprop, crd->email.type);
-			add_CardProperty(vprop, &crd->email.prop);
-	}
-
-
 
 	add_CardStrProperty(vobj, VCMailerProp, &crd->mailer);
+	
 	if (crd->timezn.prop.used) {
 		char *str;
 		
@@ -1371,6 +1366,7 @@ card_convert_to_vobject(Card *crd)
 		free(str);
 		add_CardProperty(vprop, &crd->timezn.prop);
 	}
+	
 	if (crd->geopos.prop.used) {
 		char *str;
 		
@@ -1379,16 +1375,20 @@ card_convert_to_vobject(Card *crd)
 		free(str);
 		add_CardProperty(vprop, &crd->geopos.prop);
 	}
+	
         add_CardStrProperty(vobj, VCTitleProp, &crd->title);
         add_CardStrProperty(vobj, VCBusinessRoleProp, &crd->role);
+	
 	if (crd->logo.prop.used) {
 		vprop = addPropSizedValue(vobj, VCLogoProp, 
 					  crd->logo.data, crd->logo.size);
 		add_PhotoType(vprop, crd->logo.type);
 		add_CardProperty(vprop, &crd->logo.prop);
 	}
+	
 	if (crd->agent)
 	  addVObjectProp(vobj, card_convert_to_vobject(crd->agent));
+	
 	if (crd->org.prop.used) {
 		vprop = addProp(vobj, VCOrgProp);
 		add_strProp(vprop, VCOrgNameProp, crd->org.name);
@@ -1398,8 +1398,10 @@ card_convert_to_vobject(Card *crd)
 		add_strProp(vprop, VCOrgUnit4Prop, crd->org.unit4);
 		add_CardProperty(vprop, &crd->org.prop);
 	}
+	
         add_CardStrProperty(vobj, VCCategoriesProp, &crd->categories);
         add_CardStrProperty(vobj, VCCommentProp, &crd->comment);
+	
 	if (crd->sound.prop.used) {
 		if (crd->sound.type != SOUND_PHONETIC)
 		  vprop = addPropSizedValue(vobj, VCPronunciationProp,
@@ -1411,8 +1413,10 @@ card_convert_to_vobject(Card *crd)
 		add_SoundType(vprop, crd->sound.type);
 		add_CardProperty(vprop, &crd->sound.prop);
 	}
+	
         add_CardStrProperty(vobj, VCURLProp, &crd->url);
         add_CardStrProperty(vobj, VCUniqueStringProp, &crd->uid);
+	
 	if (crd->key.prop.used) {
 		vprop = addPropValue(vobj, VCPublicKeyProp, crd->key.data);
 		add_KeyType(vprop, crd->key.type);
